!STDP v1.0: ST DATA PLOT
!STILIANOS LOUCA, 2009

!INTERFACE FOR GNUPLOT DATA-PLOTTING
!SUPPORTS 2D AND 3D DATA, ONLY SINGLE PLOTS
!PLOT APPEARANCE IS DEFINED WITH THE STDP_PLOTAPPEARANCE STRUCT
!DATA FILE IS DEFINED BY THE STDP_PLOTFILESPECIFIER STRUCT
!CALL EITHERSTDP_PLOTDATA OR STDP_PLOTFILE TO PLOT DATA IN ARRAYS AND FILE RESPECTIVELY

!TESTED ON FORTRAN 90, GNU-FORTRAN, GCC VERSION 4.5.0, GNUPLOT 4.2, MAC OS 10.5.6

module STDP
	implicit none;
	
	private :: fileExists, getNonExistingFile;
	character(len=*), parameter :: STDP_DEFAULT_FILENAME = "STDP_PlotData", STDP_DEFAULT_SCRATCH_APPENDIX="_scratch";
	
	
	integer, parameter :: STDP_INDEX_NULL=-1;
	integer, parameter :: STDP_PLOT_AS_SPLINES=0;
	integer, parameter :: STDP_PLOT_SMOOTH_UNIQUE=1;
	integer, parameter :: STDP_PLOT_AS_POINTS=2;
	integer, parameter :: STDP_PLOT_AS_LINES=3;
	integer, parameter :: STDP_PLOT_AS_LINESPOINTS=4;
	integer, parameter :: STDP_PLOT_AS_SURFACE=5;
	integer, parameter :: STDP_PLOT_AS_NET=6;
	
	integer, parameter :: STDP_POINTTYPE_DOT=0;
	integer, parameter :: STDP_POINTTYPE_CROSS=1;
	integer, parameter :: STDP_POINTTYPE_X=2;
	integer, parameter :: STDP_POINTTYPE_NULL=-1;
	
	integer, parameter :: STDP_ERROR_NONE = 0;
	integer, parameter :: STDP_ERROR_FIO = 1;
	
	type STDP_PlotAppearance
		integer :: color=1, width=1;
		integer :: plotType=STDP_PLOT_AS_POINTS;
		logical :: contours = .false.;
		integer :: pointType = STDP_POINTTYPE_NULL;
		integer :: XResolution = 200, YResolution = 200;
	end type STDP_PlotAppearance
	
	type STDP_PlotFileSpecifier
		integer :: dataColumns(3)=(/1,2,3/);
		integer :: dims=2;
		character(len=128) :: key = ""; !short title describing the data
		character(len=512) :: fileName="data";
		integer :: dataIndex = STDP_INDEX_NULL; !set to STDP_INDEX_NULL for all indices
	end type STDP_PlotFileSpecifier
	
	
contains

	logical function fileExists(file)
		character(len=*), intent(in) :: file;
		inquire(file=file, exist=fileExists);
	end function fileExists;
	
	
	recursive character(len=512) function getNonExistingFile(defaultFile) result(file)
		character(len=*), intent(in) :: defaultFile;
		integer :: i;
		i=0; file = defaultFile;
		do while(fileExists(file) .and. (i<=1e8))
			write(file, fmt="(A,I8.8)") defaultFile, i;
			i = i+1;
		end do
		if(fileExists(file)) file=getNonExistingFile(defaultFile//"STDP");
	end function getNonExistingFile
	

	type(STDP_PlotAppearance) function STDP_CreatePlotAppearance(width, color, &
		contours, plotType, pointType, XResolution, YResolution) result(appearance)
		integer, intent(in), optional :: width, color, plotType, pointType, XResolution, YResolution;
		logical, intent(in), optional :: contours;
		if(present(width)) appearance%width = max(width,1);
		if(present(color)) appearance%color = color;
		if(present(plotType)) appearance%plotType = plotType;
		if(present(contours)) appearance%contours = contours;
		if(present(pointType)) appearance%pointType = pointType;
		if(present(XResolution)) appearance%XResolution = max(2,XResolution);
		if(present(YResolution)) appearance%YResolution = max(2,YResolution);
	end function STDP_CreatePlotAppearance
	
	
	
	type(STDP_PlotFileSpecifier) function STDP_CreatePlotFileSpecifier(fileName, dims, dataColumns, index, key) result(fileSpecifier)
		character(len=*), intent(in) :: fileName;
		integer, intent(in), optional :: dims, index;
		integer, intent(in), dimension(:), optional :: dataColumns;
		character(len=*), intent(in), optional :: key;
		integer :: i;
		
		if(present(key)) fileSpecifier%key = key;
		if(present(dataColumns))then
			fileSpecifier%dataColumns = dataColumns;
			if(.not. present(dims))then
				fileSpecifier%dims = min(3, size(dataColumns));
			else
				fileSpecifier%dims = min(dims,size(dataColumns));
			end if
		else if(present(dims))then
			fileSpecifier%dims = dims;
		end if
		if(present(index))then
			fileSpecifier%dataIndex = index;
		end if
		fileSpecifier%fileName = fileName;
	end function STDP_CreatePlotFileSpecifier
	
	
	subroutine STDP_PlotData(XValues, YValues, ZValues, appearance, fileName, key, title, &
		terminal, persist, XLabel, YLabel, ZLabel, error)
		character(len=*), intent(in), optional :: fileName, terminal, key, title, XLabel, YLabel, ZLabel;
		double precision, intent(in), dimension(:), optional :: ZValues;
		double precision, intent(in), dimension(:) :: XValues, YValues;
		type(STDP_PlotAppearance), intent(in), optional :: appearance;
		integer, intent(out), optional :: error;
		logical, intent(in), optional :: persist;
		
		type(STDP_PlotFileSpecifier) :: fileSpecifier;
		integer :: fout=3, i;
		
		if(present(ZValues))then
			fileSpecifier%dims = 3;
		else
			fileSpecifier%dims = 2;
		end if
		fileSpecifier%dataColumns = (/1,2,3/);
		fileSpecifier%dataIndex = STDP_INDEX_NULL;
		if(present(fileName))then
			fileSpecifier%fileName = fileName;
		else
			fileSpecifier%fileName = getNonExistingFile(STDP_DEFAULT_FILENAME);
		end if
		if(present(key)) fileSpecifier%key=key;
		
		open(unit=fout, file=fileSpecifier%fileName, status="replace", action="write", err=910);

		if(present(ZValues))then
			do i=1,size(XValues)
				write(fout,*, err=910) XValues(i),YValues(i),ZValues(i);
			end do
		else
			do i=1,size(XValues)
				write(fout,*, err=910) XValues(i),YValues(i);
			end do
		end if
		close(fout);
		
		call STDP_PlotFile(fileSpecifier=fileSpecifier,&
			appearance = appearance, terminal=terminal, persist = persist, &
			XLabel = XLabel, YLabel = YLabel, ZLabel = ZLabel, title=title, error = error);
		return;
		
910		if(present(error)) error = STDP_ERROR_FIO;
	end subroutine STDP_PlotData




	subroutine STDP_PlotFile(fileSpecifier, appearance, minX, maxX, minY, maxY, minZ, maxZ,&
		terminal, persist, XLabel, YLabel, ZLabel, title, error)
		double precision, intent(in), optional :: minX, maxX, minY, maxY, minZ, maxZ;
		character(len=*), intent(in), optional :: terminal, XLabel, YLabel, ZLabel, title;
		type(STDP_PlotAppearance), intent(in), optional :: appearance;
		type(STDP_PlotFileSpecifier), intent(in) :: fileSpecifier;
		logical, intent(in), optional :: persist;
		integer, intent(out), optional :: error;
		
		character(len=128) :: plotType, plotCommand, key, rangeX,rangeY,rangeZ,usingAxis,index,persistL,pointType;
		type(STDP_PlotAppearance) :: appearanceL;
		character(len=512) :: fileScratchName;
		integer :: fileScratch=2;
		logical :: asGrid, hide3D, showKeys;
		
		if(present(appearance))then
			appearanceL = appearance;
		else
			appearanceL = STDP_CreatePlotAppearance();
		end if
		
		!Point type
		select case(appearanceL%pointType)
		case(STDP_POINTTYPE_DOT)
			pointType = "pt 0";
		case(STDP_POINTTYPE_CROSS)
			pointType = "pt 1";
		case(STDP_POINTTYPE_X)
			pointType = "pt 2";
		case default !or POINT_TYPE_NULL
			pointType = "";
		end select
		!Plot Type
		if(fileSpecifier%dims == 3)then
			write(usingAxis,*) "using ",fileSpecifier%dataColumns(1),":",fileSpecifier%dataColumns(2),":",fileSpecifier%dataColumns(3);
			plotCommand = "splot";
			select case(appearanceL%plotType)
			case(STDP_PLOT_AS_LINES)
				write(plotType,*) "with lines lt ",appearanceL%color," lw ",appearanceL%width;
				asGrid = appearanceL%contours;
				hide3D = .true.;
			case(STDP_PLOT_AS_LINESPOINTS)
				write(plotType,*) "with linespoints ",trim(pointType)," lt ",appearanceL%color," lw ",appearanceL%width;
				asGrid = appearanceL%contours;
				hide3D = .true.;
			case(STDP_PLOT_AS_POINTS)
				write(plotType,*) "with points ",trim(pointType)," lt ",appearanceL%color," ps ",appearanceL%width;
				asGrid = appearanceL%contours;
				hide3D = .false.;
			case(STDP_PLOT_AS_SURFACE)
				write(plotType, *) "with pm3d";
				asGrid = .true.;
				hide3D = .true.;
			case(STDP_PLOT_AS_NET)
				write(plotType, *) "with lines";
				asGrid = .true.;
				hide3D = .true.;
			case default
				if(appearanceL%contours)then !user probably would want surface-plot
					write(plotType, *) "with lines lt ",appearanceL%color," lw ",appearanceL%width;
					asGrid = .true.;
					hide3D = .true.;
				else !just plot single points
					write(plotType, *) "with points ",trim(pointType)," lt ",appearanceL%color," ps ",appearanceL%width;
					asGrid = .false.;
					hide3D = .false.;
				end if
			end select
		else
			write(usingAxis,*) "using ",fileSpecifier%dataColumns(1),":",fileSpecifier%dataColumns(2);
			plotCommand = "plot";
			select case(appearanceL%plotType)
			case(STDP_PLOT_AS_SPLINES)
				write(plotType,*) "smooth csplines lt ",appearanceL%color," lw ",appearanceL%width;
			case(STDP_PLOT_AS_LINES)
				write(plotType,*) "with lines lt ",appearanceL%color," lw ",appearanceL%width;
			case(STDP_PLOT_AS_LINESPOINTS)
				write(plotType,*) "with linespoints ",trim(pointType)," lt ",appearanceL%color," lw ",appearanceL%width;
			case(STDP_PLOT_AS_POINTS)
				write(plotType,*) "with points ",trim(pointType)," lt ",appearanceL%color," ps ",appearanceL%width;
			case default
				write(plotType,*) "smooth unique lt ",appearanceL%color," lw ",appearanceL%width;
			end select
			asGrid = .false.;
			hide3D = .false.;
		end if
	
		!Title
		if(trim(fileSpecifier%key) /= "") then
			write(key,*) "title '",trim(fileSpecifier%key),"'";
			showKeys = .true.;
		else
			key = "";
			showKeys = .false.;
		end if
		!Ranges
		if(present(minX) .and. present(maxX))then
			write(rangeX,*) "[",minX,":",maxX,"]";
		else if(present(minX) .and. (.not. present(maxX)))then
			write(rangeX,*) "[",minX,":]";
		else if((.not. present(minX)) .and. present(maxX))then
			write(rangeX,*) "[:",maxX,"]";
		else
			rangeX = "";
		end if
		if(present(minY) .and. present(maxY))then
			write(rangeY,*) "[",minY,":",maxY,"]";
		else if(present(minY) .and. (.not. present(maxY)))then
			write(rangeY,*) "[",minY,":]";
		else if((.not. present(minY)) .and. present(maxY))then
			write(rangeY,*) "[:",maxY,"]";
		else
			rangeY = "";
		end if
		if(present(minZ) .and. present(maxZ))then
			write(rangeZ,*) "[",minZ,":",maxZ,"]";
		else if(present(minZ) .and. (.not. present(maxZ)))then
			write(rangeZ,*) "[",minZ,":]";
		else if((.not. present(minZ)) .and. present(maxZ))then
			write(rangeZ,*) "[:",maxZ,"]";
		else
			rangeZ = "";
		end if
		!Data Index
		if(fileSpecifier%dataIndex == STDP_INDEX_NULL)then
			index = "";
		else
			write(index,*) "index ",fileSpecifier%dataIndex;
		end if
		!Persist?
		persistL = "persist";
		if(present(persist))then
			if(.not. persist) persistL = "";
		end if
		
		
		!Write Gnuplot command file
		fileScratchName = getNonExistingFile(trim(fileSpecifier%fileName) // STDP_DEFAULT_SCRATCH_APPENDIX);
		open(unit=fileScratch, file=trim(fileScratchName), status="replace", action="write", err = 911);
		!set term
		if(present(terminal))then
			if(terminal /= "aqua")then
				write(fileScratch,*, err=911) "set term ",trim(terminal)," ",trim(persistL);
			else 
				write(fileScratch,*, err=911) "set term ",trim(terminal);
			end if
		else
			write(fileScratch,*, err=911) "set term x11 ",trim(persistL);
		end if
		!set axis labels
		if(present(XLabel))then
			write(fileScratch, *, err=911) "set xlabel '",trim(XLabel),"'";
		else
			write(fileScratch, *, err=911) "set xlabel 'X'";
		end if
		if(present(YLabel))then
			write(fileScratch, *, err=911) "set ylabel '",trim(YLabel),"'";
		else
			write(fileScratch, *, err=911) "set ylabel 'Y'";
		end if
		if(present(ZLabel))then
			write(fileScratch, *, err=911) "set zlabel '",trim(ZLabel),"'";
		else
			write(fileScratch, *, err=911) "set zlabel 'Z'";
		end if
		!set 3D-parameters
		if(hide3D)then
			write(fileScratch,*, err=911) "set hidden3d";
		end if
		if(asGrid)then
			write(fileScratch, *, err=911) "set dgrid3d ",appearanceL%YResolution,", ",appearanceL%XResolution;
		end if
		!set contours
		if(fileSpecifier%dims == 3)then
			if(appearanceL%contours)then
				write(fileScratch, *, err=911) "set contours";
				write(fileScratch, *, err=911) "set cntrparam bspline";
			else
				write(fileScratch, *, err=911) "unset contours";
			end if
		end if
		!set resolution
		write(fileScratch, *, err=911) "set isosamples ",appearanceL%XResolution, ", ", appearanceL%YResolution;
		write(fileScratch, *, err=911) "set samples ",appearanceL%XResolution, ", ", appearanceL%YResolution;
		!set key position
		if(showKeys)then
			write(fileScratch, *, err=911) "set key under";
		else
			write(fileScratch, *, err=911) "unset key";
		end if
		!set title
		write(fileScratch, *, err=911) "set title '",trim(title),"'";
		!write plot command
		write(fileScratch, *, err=911) trim(plotCommand)," ",trim(rangeX), trim(rangeY), trim(rangeZ), " '", &
			trim(fileSpecifier%fileName), "' ", trim(index)," ", trim(usingAxis)&
			, " ", trim(plotType), " ", trim(key);
		close(fileScratch);
		
		!Call Gnuplot
		call system("gnuplot '"//trim(fileScratchName)//"'");
		call system("rm '"//trim(fileScratchName)//"'");
		if(present(error)) error = STDP_ERROR_NONE;
		return; 
		
911		if(present(error)) error = STDP_ERROR_FIO;
	end subroutine STDP_PlotFile

end module STDP
